// DePulse 2005/04/10
//         2005/04/11 update

#include <windows.h>
#include "avisynth.h"
#include <stdio.h>



class DePulse : public GenericVideoFilter {
public:
	DePulse(PClip _child, int h, int l, int d, bool debug, IScriptEnvironment* env);
	~DePulse();
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
	int _h;
	int _l;
	int _d;
	bool _debug;
};

DePulse::DePulse(PClip _child, int h, int l, int d, bool debug, IScriptEnvironment* env) : GenericVideoFilter(_child) {
    if (!vi.IsYUY2())
        env->ThrowError("DePulse: requires YUY2 input.");
    (h>255) ? _h=255 : _h=h;
    (l>255) ? _l=255 : _l=l;
    (d>255) ? _d=255 : _d=d;
    (h<0) ? _h=0 : _h=h;
    (l<0) ? _l=0 : _l=l;
    (d<0) ? _d=0 : _d=d;
    _debug=debug;
}

DePulse::~DePulse() {
}

PVideoFrame __stdcall DePulse::GetFrame(int n, IScriptEnvironment* env) {
    //*** in-place filtering ***
    PVideoFrame src = child->GetFrame(n, env);
    PVideoFrame srcbf = child->GetFrame( ((n==0) ? n : n-1) , env);
    PVideoFrame srcaf = child->GetFrame( ((n==vi.num_frames-1) ? n : n+1) , env);
    env->MakeWritable(&src);
    const int   pitch   = src->GetPitch();
    const int   rowsize = src->GetRowSize();
    const int   height  = src->GetHeight();
    const int   modulo  = pitch - src->GetRowSize();

    int lo;
    int hi;
    unsigned int col0;
    unsigned int col1;
    unsigned int col2;
    unsigned int col3;
    unsigned int col4;
    unsigned int col5;
    unsigned int col6;
    unsigned int col7;
    unsigned int col8;


    const BYTE *srcp    = src->GetReadPtr();
    const BYTE *srcpbf    = srcbf->GetReadPtr();
    const BYTE *srcpaf    = srcaf->GetReadPtr();
    BYTE       *dstp    = src->GetWritePtr();
    
    dstp += pitch;

    for (int j=1; j<height-1; j+=2) {
        for (int i=0; i<rowsize; i+=4) {

col0=(int)srcp[j*pitch+i];
col1=(int)srcp[j*pitch+i-pitch];
col2=(int)srcp[j*pitch+i+pitch];
col3=(int)srcpaf[j*pitch+i-pitch];
col4=(int)srcpaf[j*pitch+i+pitch];
col5=(int)srcpbf[j*pitch+i];
col6=(int)srcpaf[j*pitch+i];
col7=(int)srcp[j*pitch+i-pitch-((j==1) ? 0 : pitch)];
col8=(int)srcp[j*pitch+i+pitch+((j==height-3) ? 0 : pitch)];
lo=col0-_d;

            if ( (col0>_h)&&(col1<lo)&&(col2<lo)&&(col3<lo)&&(col4<lo)&&(col5<lo)&&(col6<lo)&&(col7<lo)&&(col8<lo) ) {
                dstp[i]=(col1+col2+col3+col4+col5+col6+col7+col8)/8;
                if (_debug) {
                    dstp[i+1]=250;
                    dstp[i+3]=250;
                }
            }

hi=col0+_d;

            if ( (col0<_l)&&(col1>hi)&&(col2>hi)&&(col3>hi)&&(col4>hi)&&(col5>hi)&&(col6>hi)&&(col7>hi)&&(col8>hi) ) {
                dstp[i]=(col1+col2+col3+col4+col5+col6+col7+col8)/8;
                if (_debug) {
                    dstp[i+1]=10;
                    dstp[i+3]=10;
                }
            }

col0=(int)srcp[j*pitch+i+2];
col1=(int)srcp[j*pitch+i-pitch+2];
col2=(int)srcp[j*pitch+i+pitch+2];
col3=(int)srcpaf[j*pitch+i-pitch+2];
col4=(int)srcpaf[j*pitch+i+pitch+2];
col5=(int)srcpbf[j*pitch+i+2];
col6=(int)srcpaf[j*pitch+i+2];
col7=(int)srcp[j*pitch+i+2-pitch-((j==1) ? 0 : pitch)];
col8=(int)srcp[j*pitch+i+2+pitch+((j==height-3) ? 0 : pitch)];
lo=col0-_d;

            if ( (col0>_h)&&(col1<lo)&&(col2<lo)&&(col3<lo)&&(col4<lo)&&(col5<lo)&&(col6<lo)&&(col7<lo)&&(col8<lo) ) {
                dstp[i+2]=(col1+col2+col3+col4+col5+col6+col7+col8)/8;
                if (_debug) {
                    dstp[i+1]=250;
                    dstp[i+3]=250;
                }
            }

hi=col0+_d;

            if ( (col0<_l)&&(col1>hi)&&(col2>hi)&&(col3>hi)&&(col4>hi)&&(col5>hi)&&(col6>hi)&&(col7>hi)&&(col8>hi) ) {
                dstp[i+2]=(col1+col2+col3+col4+col5+col6+col7+col8)/8;
                if (_debug) {
                    dstp[i+1]=10;
                    dstp[i+3]=10;
                }
            }

        }
        dstp += pitch;

        for (int i=0; i<rowsize; i+=4) {

col0=(int)srcp[(j+1)*pitch+i];
col1=(int)srcp[(j+1)*pitch+i-pitch];
col2=(int)srcp[(j+1)*pitch+i+pitch];
col3=(int)srcpbf[(j+1)*pitch+i-pitch];
col4=(int)srcpbf[(j+1)*pitch+i+pitch];
col5=(int)srcpbf[(j+1)*pitch+i];
col6=(int)srcpaf[(j+1)*pitch+i];
col7=(int)srcp[(j+1)*pitch+i-pitch-((j==1) ? 0 : pitch)];
col8=(int)srcp[(j+1)*pitch+i+pitch+((j==height-3) ? 0 : pitch)];
lo=col0-_d;

            if ( (col0>_h)&&(col1<lo)&&(col2<lo)&&(col3<lo)&&(col4<lo)&&(col5<lo)&&(col6<lo)&&(col7<lo)&&(col8<lo) ) {
                dstp[i]=(col1+col2+col3+col4+col5+col6+col7+col8)/8;
                if (_debug) {
                    dstp[i+1]=250;
                    dstp[i+3]=250;
                }
            }

hi=col0+_d;

            if ( (col0<_l)&&(col1>hi)&&(col2>hi)&&(col3>hi)&&(col4>hi)&&(col5>hi)&&(col6>hi)&&(col7>hi)&&(col8>hi) ) {
                dstp[i]=(col1+col2+col3+col4+col5+col6+col7+col8)/8;
                if (_debug) {
                    dstp[i+1]=10;
                    dstp[i+3]=10;
                }
            }

col0=(int)srcp[(j+1)*pitch+i+2];
col1=(int)srcp[(j+1)*pitch+i-pitch+2];
col2=(int)srcp[(j+1)*pitch+i+pitch+2];
col3=(int)srcpbf[(j+1)*pitch+i-pitch+2];
col4=(int)srcpbf[(j+1)*pitch+i+pitch+2];
col5=(int)srcpbf[(j+1)*pitch+i+2];
col6=(int)srcpaf[(j+1)*pitch+i+2];
col7=(int)srcp[(j+1)*pitch+i+2-pitch-((j==1) ? 0 : pitch)];
col8=(int)srcp[(j+1)*pitch+i+2+pitch+((j==height-3) ? 0 : pitch)];
lo=col0-_d;

            if ( (col0>_h)&&(col1<lo)&&(col2<lo)&&(col3<lo)&&(col4<lo)&&(col5<lo)&&(col6<lo)&&(col7<lo)&&(col8<lo) ) {
                dstp[i+2]=(col1+col2+col3+col4+col5+col6+col7+col8)/8;
                if (_debug) {
                    dstp[i+1]=250;
                    dstp[i+3]=250;
                }
            }

hi=col0+_d;

            if ( (col0<_l)&&(col1>hi)&&(col2>hi)&&(col3>hi)&&(col4>hi)&&(col5>hi)&&(col6>hi)&&(col7>hi)&&(col8>hi) ) {
                dstp[i+2]=(col1+col2+col3+col4+col5+col6+col7+col8)/8;
                if (_debug) {
                    dstp[i+1]=10;
                    dstp[i+3]=10;
                }
            }

        }
        dstp += pitch;
    }

    return src;
}

AVSValue __cdecl Create_DePulse(AVSValue args, void* user_data, IScriptEnvironment* env) {
    return new DePulse(args[0].AsClip(), args[1].AsInt(180), args[2].AsInt(60), args[3].AsInt(30), args[4].AsBool(false), env);  
}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
    env->AddFunction("DePulse", "c[h]i[l]i[d]i[debug]b", Create_DePulse, 0);
    return "`DePulse' DePulse plugin";
}
